function M = fdetect_hn(I, Net)
% FDETECT_H face detection hiararchical

AverageT = 400;
Dx = [-1 -2 -1; 0 0 0; 1 2 1];
Dy = [-1 0 1; -2 0 2; -1 0 1];

NoF = 20;   % Number of posible face in the Image
MaxFaceNum = 5;
Ratio = 0.89;
Net.threshold = 0.5;

[Height, Width] = size(I);
MinFaceSize = Width / 10;
if(MinFaceSize<20)
   MinFaceSize = 20;
end
MaxFaceSize = Width / 2;

MaxFaceSize = 200;
MinFaceSize = 100;

ellip = double(imread('ellip.bmp'));
ellipx = abs(conv2(ellip, Dx, 'valid'));
ellipy = abs(conv2(ellip, Dy, 'valid'));
ellipx = ellipx .* 255 ./ max(max(ellipx));
ellipy = ellipy .* 255 ./ max(max(ellipy));

FaceSize = MinFaceSize;
temp = 20/MinFaceSize;
pN = 1;
while(FaceSize <= MaxFaceSize)
   fprintf(1, 'Detect Face Size: %d\n', floor(FaceSize) );
   SmallImage = imresize(I, temp);
   % Detection
   Gx = abs(conv2(SmallImage, Dx, 'valid'));
   Gx = Gx .* 255 ./ max(max(Gx));
   Gy = abs(conv2(SmallImage, Dy, 'valid'));
   Gy = Gy .* 255 ./ max(max(Gy));
   Rx = conv2(Gx, ellipx, 'same');
   Ry = conv2(Gy, ellipy, 'same');
   R = Rx + Ry;
   pEnd = pN+NoF-1;
   [pV(pN:pEnd), pY(pN:pEnd), pX(pN:pEnd)] = maxn(R, NoF, 0);
   pY(pN:pEnd) = pY(pN:pEnd) / temp;
   pX(pN:pEnd) = pX(pN:pEnd) / temp;
   pSize(pN:pEnd) = FaceSize;
   pN = pEnd + 1;
   
   temp = temp * Ratio;
   FaceSize = 20/temp;
end

NewpV = [];
k = 1;
for i=1:length(pV)
   CenterX = pX(i);
   CenterY = pY(i);
   HalfWidth = pSize(i) / 2;
   HalfHeight = HalfWidth * 1.2;
   ltx(k) = floor(CenterX - HalfWidth);
   lty(k) = floor(CenterY - HalfHeight);
   rbx(k) = floor(CenterX + HalfWidth);
   rby(k) = floor(CenterY + HalfHeight);
   if( ltx(k) < 1 | lty(k) < 1 | rbx(k) > Width | rby(k) > Height )
      ltx(k) = [];
      lty(k) = [];
      rbx(k) = [];
      rby(k) = [];
   else
      Vector = To460(I, lty(k), ltx(k), rbx(k)-ltx(k));
      if( strcmp(Net.mode, 'std') )
         Vector = trastd(Vector, Net.mean, Net.std);
      elseif( strcmp(Net.mode, 'pca') )
         Vector = trastd(Vector, Net.mean, Net.std);
         Vector = trapca(Vector, Net.transMat);
      end
      IsFace = sim(Net.net, Vector);
      if(IsFace > Net.threshold)
         NewpV(k) = pV(i);
         k = k+1;
      else
         ltx(k) = [];
         lty(k) = [];
         rbx(k) = [];
         rby(k) = [];
      end
   end
end

if( length(NewpV) > 0 )
   [Value, Y, X] = maxn(NewpV, MaxFaceNum, 0);
   for i=1:length(X)
      NewLtx = ltx(X(i)); NewLty = lty(X(i)); NewRbx = rbx(X(i)); NewRby = rby(X(i));
   end
   M = imrect(I,NewLtx,NewLty,NewRbx,NewRby);
else
   M = I;
end
imshow(M);

function Im460 = To460(Im,y,x,Width)
% To16x16 
Ratio = Width/20;
Step = floor(Ratio);
p = 0;
for k=1:23
   i = round( y + (k-1)*Ratio );
   for m=1:20
      j = round( x + (m-1)*Ratio );
      p = p + 1;
      Im460(p,1) = mean(mean(Im(i:i+Step,j:j+Step)));
   end
end
